import requests,json


remote_server = "http://llm.aidatatools.com"
#emote_server = "http://localhost:5000"

def send_sysinfo(data_dict):
    
    json_object = json.dumps(data_dict, indent = 4)
    print(json_object)
    print('-'*10)

    url = f'{remote_server}/api/receiver.php'

    x = requests.post(url, data = json_object)

    return x.text 

def send_benchmark(uuid, ollama_version,data_dict):
    data_dict.update({"uuid":f"{uuid}"})
    data_dict.update({"ollama_version":f"{ollama_version}"})
    json_object = json.dumps(data_dict, indent = 4)
    print(json_object)
    print('-'*10)

    url = f'{remote_server}/api/receiver_benchmark.php'

    x = requests.post(url, data = json_object)

    return x.text 


import argparse
import yaml
import subprocess
import ollama

parser = argparse.ArgumentParser(
    prog="python3 check_models.py",
    description="Before running check_models.py, please make sure you installed ollama successfully \
        on macOS, Linux, or WSL2 on Windows. You can check the website: https://ollama.ai",
    epilog="Author: Jason Chuang")

parser.add_argument("-v",
                    "--verbose",
                    action="store_true",
                    help="this program helps you check whether you have ollama benchmark models installed")

parser.add_argument("-m",
                    "--models",
                    type=str,
                    help="provide benchmark models YAML file path. ex. ../data/benchmark_models.yml")

def parse_yaml(yaml_file_path):
    with open(yaml_file_path, 'r') as stream:
        try:
            data=yaml.safe_load(stream)
            #print(d)
        except yaml.YAMLError as e:
            print(e)
    return data

def pull_models(models_file_path):
    print(f"Checking and pulling the following LLM models")
    models_dict = parse_yaml(models_file_path)
    for x in models_dict['models']:
        model_name = x['model']
        print(model_name)
        ollama.pull(model_name)

if __name__ == "__main__": 
    args = parser.parse_args()
    #print(f"args.verbose value：{args.verbose}")
    if (args.models is not None):
        print(f"args.models file path：{args.models}")
        models_dict = parse_yaml(args.models)
        for x in models_dict['models']:
            model_name = x['model']
            print(model_name)
            result = subprocess.run(['ollama', 'pull', model_name], stdout=subprocess.PIPE)
            result.stdout
    


import typer
import pkg_resources
import speedtest
import time
from .systeminfo import sysmain
from .security_connection import connection
from llm_bench import check_models, check_ollama, run_benchmark

app = typer.Typer()

def get_model_path(model: str) -> str:
    """ Helper function to return the correct file path based on the model size """
    return pkg_resources.resource_filename('llm_bench', f'data/{model}_models.yml')

@app.command()
def check_internet():
    """Check and print the internet speed."""
    try:
        start_time = time.time()
        st = speedtest.Speedtest()
        st.get_best_server()
        download_speed = st.download() / 1_000_000
        upload_speed = st.upload() / 1_000_000
        elapsed_time = time.time() - start_time
        print(f"Download Speed: {download_speed:.2f} Mbps")
        print(f"Upload Speed: {upload_speed:.2f} Mbps")
        print(f"Internet speed test duration: {elapsed_time:.2f} seconds")
    except speedtest.ConfigRetrievalError as e:
        print(f"Failed to retrieve speed test configuration: {e}")
    except speedtest.SpeedtestException as e:
        print(f"Failed to complete the speed test: {e}")
    except Exception as e:
        print(f"An unexpected error occurred: {e}")

@app.command()
def sysinfo(ollamabin: str = typer.Option('ollama', help="Path to the Ollama binary.")):
    sys_info = sysmain.get_extra()
    print(f"Total memory size : {sys_info['memory']:.2f} GB") 
    print(f"cpu_info: {sys_info['cpu']}")
    print(f"gpu_info: {sys_info['gpu']}")
    print(f"os_version: {sys_info['os_version']}")

    ollama_version = check_ollama.check_ollama_version(ollamabin)
    print(f"ollama_version: {ollama_version} \n")

@app.command()
def run(
    ollamabin: str = typer.Option('ollama', help="Path to the Ollama binary."),
    model: str = typer.Option(
        "all", help="Select model: all-small, gemma-small, llama3-small, mistral-small, phi3-small, qwen-small, all-medium, gemma-medium, phi3-medium, llama3-medium, all-large, drbx-large, llama3-large",
        case_sensitive=False, show_choices=True
    ),
    test: bool = typer.Option(
        False, '--test', help="Flag to toggle between default and alternative benchmark tests."
    ),
    steps: int = typer.Option(
        10, help="Set the amount of inferences",
        min=1, max=10
    )
):
    """Run the benchmark tests."""
    benchmark_file = pkg_resources.resource_filename('llm_bench', 'data/benchmark_instructions.yml')
    models_file = get_model_path(model)

    start_time = time.time()
    check_models.pull_models(models_file)
    elapsed_time = time.time() - start_time
    print(f"Model pulling time: {elapsed_time:.2f} seconds")
    print('-' * 10)

    if test:
        print(f"Testing pulled model(s)")

    run_benchmark.run_benchmark(models_file, steps, benchmark_file, test, ollamabin)

if __name__ == "__main__":
    app()



import requests

headers = {
    'Content-Type': 'application/json',
}
#"model" : "phi"
#"model" : "llama2"
#"model" : "mistral"

data = '{\n  "model": "llama2:7b",\n  "prompt": "Why is the sky blue? Is it related to sunshine?",\n  "stream": false\n}'

response = requests.post('http://localhost:11434/api/generate', headers=headers, data=data)
jsonResponse = response.json()

model = ""
total_duration = 0.0
load_duration = 0.0
prompt_eval_count = 0
prompt_eval_duration = 0.0
eval_count = 0
eval_duration = 0.0
for key, value in jsonResponse.items():
    if (key=="response"):
        pass
    elif (key=="context"):
        pass
    elif (key=="model"):
        model = value
    elif (key=="total_duration"):
        total_duration = float(value)/(10**6)
    elif (key=="load_duration"):
        load_duration = float(value)/(10**6)
    elif (key=="prompt_eval_count"):
        prompt_eval_count=int(value)
    elif (key=="prompt_eval_duration"):
        prompt_eval_duration=float(value)/(10**6)
    elif (key=="eval_count"):
        eval_count=int(value)
    elif (key=="eval_duration"):
        eval_duration=float(value)/(10**6)

print(f"model = {model}")

print(f"{'total_duration time': >20} = {total_duration:10.2f} ms")
print(f"{'load_duration time': >20} = {load_duration:10.2f} ms")

print(f"{'prompt eval time ': >20} = {prompt_eval_duration:10.2f} ms / {prompt_eval_count:>6} tokens")    
print(f"{'eval time ': >20} = {eval_duration:10.2f} ms / {eval_count:>6} tokens ")
print(f"Performance: {eval_count/eval_duration*1000:10.2f}(tokens/s)")    
         



import platform
import psutil
import GPUtil
import subprocess
import os
import uuid
import base64

def get_total_memory_size():
    memory_info = psutil.virtual_memory()
    return (memory_info.total/(2**(10*3)))

def get_system_info():
    #print("System Information:")
    system_info = platform.uname()
    ans={}
    ans['system'] = f"{system_info.system}"
    ans['node_name'] = f"{system_info.node}"
    ans['release'] = f"{system_info.release}"
    ans['version'] = f"{system_info.version}"
    ans['machine'] = f"{system_info.machine}"
    ans['processor'] = f"{system_info.processor}"
    return ans

def get_cpu_info():
    cpu_info = platform.processor()
    cpu_count = psutil.cpu_count(logical=False)
    logical_cpu_count = psutil.cpu_count(logical=True)

    #print("\nCPU Information:")
    ans = {}
    ans['processor'] = f"{cpu_info}"
    ans['physical_cores'] = f"{cpu_count}"
    ans['logical_cores'] = f"{logical_cpu_count}"
    return ans

def get_memory_info():
    memory_info = psutil.virtual_memory()

    #print("\nMemory Information: Unit : bytes")
    ans={}
    ans['total_memory'] = f"{memory_info.total}"
    ans['available_memory'] = f"{memory_info.available}"
    ans['used_memory'] = f"{memory_info.used}"
    ans['memory_utilization'] = f"{memory_info.percent:.2f}%"
    return ans

def get_disk_info():
    disk_info = psutil.disk_usage('/')

    #print("\nDisk Information: Unit : bytes")
    ans={}
    ans['total_disk_space'] = f"{disk_info.total}"
    ans['used_disk_space'] = f"{disk_info.used}"
    ans['free_disk_space'] = f"{disk_info.free}"
    ans['disk_space_utilization'] = f" {disk_info.percent:.2f}%"

#Only Nvidia GPU on Windows and Linux
def get_gpu_info():    

    ans={}
    if (get_system_info()['system']=='Darwin'):
        ans['0'] = "no_gpu"
        return ans
    else:
        gpus = GPUtil.getGPUs()
    
    if not gpus:
        print("\nNo GPU detected.")
        ans['0'] = "no_gpu"
    else:
        ans['0'] = "there_is_gpu"

        for i, gpu in enumerate(gpus):
            #print(f"\nGPU {i + 1} Information:")
            ans[f'{i+1}'] = {}
            ans[f'{i+1}']['id'] = f"{gpu.id}"
            ans[f'{i+1}']['name'] = f"{' '.join(gpu.name.splitlines())}"
            ans[f'{i+1}']['driver'] = f"{gpu.driver}"
            ans[f'{i+1}']['gpu_memory_total'] = f"{gpu.memoryTotal} MB"
            ans[f'{i+1}']['gpu_memory_free'] = f"{gpu.memoryFree} MB"
            ans[f'{i+1}']['gpu_memory_used'] = f"{gpu.memoryUsed} MB"
            ans[f'{i+1}']['gpu_load'] = f"{gpu.load*100}%"
            ans[f'{i+1}']['gpu_temperature'] = f"{gpu.temperature}°C"
    
    return ans

def check_windows_shell():
    parent_pid = os.getppid()
    shell_name = psutil.Process(parent_pid).name().lower()
    if 'cmd' in shell_name:
        return 'CMD'
    elif 'powershell' in shell_name:
        return 'PowerShell'
    else:
        return 'Unknown'


def get_extra():
    system_info = platform.uname()
    ans={}
    ans['system'] = f"{system_info.system}"
    ans['memory'] = get_total_memory_size()
    ans['cpu'] = f"unknown"
    ans['gpu'] = f"unknown"
    ans['os_version'] = f"unknown"

    try:
        if(system_info.system=='Darwin'):
            ans['system_name'] = "macOS"

            print('----------Apple Mac---------')
            r1 = subprocess.run(['system_profiler', 'SPHardwareDataType'],capture_output=True,text=True)
            #ans['hardware'] = f"{r1.stdout}"
            for line in r1.stdout.split('\n'):
                if('Model Identifier' in line):
                    ans['model']=f"{line[24:]}"
                    

            for line in r1.stdout.split('\n'):
                if ('Chip' in line):
                    ans['cpu']=f"{line[12:]}"
            
            if(ans['cpu'].startswith('Apple')):
                ans['gpu'] = ans['cpu']
            else:
                ans['gpu'] = 'no_gpu'
            
            #r2 = subprocess.run(['system_profiler', 'SPDisplaysDataType'],capture_output=True,text=True)
            #ans['display'] = f"{r2.stdout}"
            r3 = subprocess.run(['system_profiler', 'SPSoftwareDataType'],capture_output=True,text=True)
            #ans['software'] = f"{r3.stdout}"

            for line in r3.stdout.split('\n'):
                if ('System Version' in line):
                    ans['os_version']=f"{line[22:]}"
            return ans
        elif(system_info.system=='Linux'):
            ans['system_name'] = "Linux"

            print('-------Linux----------')

            try:     
                r2 = subprocess.run(['lsb_release','-a'],capture_output=True,text=True)
                software = f"{r2.stdout}"
                for line in software.split('\n'):
                    if ('Description' in line):
                        ans['os_version']=f"{line[12:]}".strip()
            except:
                r2 = subprocess.run(['cat','/etc/os-release'],capture_output=True,text=True)
                software = f"{r2.stdout}"
                for line in software.split('\n'):
                    if ('PRETTY_NAME' in line):
                        ans['os_version']=f"{line[12:]}".strip()
            
            try:
                r1 = subprocess.run(['lshw','-C','cpu'],capture_output=True,text=True)
                for line in r1.stdout.split('\n'):
                    if ('product' in line):
                        ans['cpu']=f"{line[16:]}"
            except:
                cmd = ['lscpu']
                ps = subprocess.Popen(cmd, stdout=subprocess.PIPE)
                cmd = ['grep', 'Model name']
                grep = subprocess.Popen(cmd, stdin=ps.stdout, stdout=subprocess.PIPE,
                        encoding='utf-8')
                ps.stdout.close()
                output, _ = grep.communicate()
                python_processes = output.split('\n')
                ans['cpu'] = python_processes[0][16:].strip() 

            if (get_gpu_info()['0']=='no_gpu'):
                try:
                    r2 = subprocess.run(['lshw','-C','display'],capture_output=True,text=True)
                    for line in r2.stdout.split('\n'):
                        if ('product' in line):
                            ans['gpu']=f"{line[16:]}"
                except:
                    ans['gpu']="no_gpu"
            else :
                print(f"{get_gpu_info()['1']}")
                try:
                     print(get_gpu_info()['2'])
                     print('At least two GPU cards')
                except:
                     print('Only one GPU card')
                #List only the first gpu name
                ans['gpu']=get_gpu_info()['1']['name']

                                        
            return ans
            
        elif(system_info.system=='Windows'):
            ans['system_name'] = "Windows"

            prefix_exe='powershell.exe'
            #print("Python is running in:", check_windows_shell(), "on Windows")
            ans['run_in'] = f"{check_windows_shell()}"           
                        
            r1 = subprocess.run([prefix_exe,'Get-WmiObject','Win32_Processor'],capture_output=True,text=True)
            #print(r1.stdout)
            r_cpu = subprocess.run([prefix_exe,'(Get-WmiObject Win32_Processor).Name'],capture_output=True,text=True)
            #print(r_cpu.stdout)
            ans['cpu']=r_cpu.stdout.strip()
            r2 = subprocess.run([prefix_exe,'Get-WmiObject','Win32_PhysicalMemory'],capture_output=True,text=True)
            #print(r2.stdout)
            r3 = subprocess.run([prefix_exe,'Get-WmiObject','Win32_VideoController'],capture_output=True,text=True)
            #print(r3.stdout)
            r_gpu = subprocess.run([prefix_exe,'(Get-WmiObject Win32_VideoController).Caption'],capture_output=True,text=True)
            str_gpu = r_gpu.stdout.strip()
            str_gpu = ' '.join(str_gpu.splitlines())
            #print(r_gpu)
            ans['gpu'] = str_gpu
            r4 = subprocess.run([prefix_exe,'(Get-WmiObject Win32_OperatingSystem).Caption'],capture_output=True,text=True)
            #print(r4.stdout)
            ans['os_version'] = f"{r4.stdout}"
        
        return ans

    except:
        print("error! when retrieving os_version, cpu, or gpu !")
    
    return ans

def get_uuid():
    sys_info = get_extra()
    system_name = sys_info['system_name']
    cpu = sys_info['cpu']
    gpu  = sys_info['gpu']
    memory = f"{sys_info['memory']:.2f}"

    id_str = f"{system_name}|{cpu}|{gpu}|{memory}"
    #print(id_str)
    uuid5 = uuid.uuid5(uuid.NAMESPACE_X500, id_str)
    return uuid5    

if __name__ == "__main__":
    #sysinfo = get_extra()
    uuid5 = get_uuid()
    print("UUID version 5:", uuid5)
    




import subprocess

def run_command(command):
    try:
        result = subprocess.run(command, capture_output=True, text=True, check=True)
        return result.stdout.strip()  # Return the output of the command
    except subprocess.CalledProcessError as e:
        print(f"Error executing command '{command}': {e}")
        return None

def check_ollama_version(ollamabin='ollama'):
    res = run_command([ollamabin, '--version'])
    ans = res.split('\n')
    #print(ans[-1])
    if("warning" in ans[-1].lower()):
        return ans[-1][27:]
    else:
        return ans[-1][18:]

if __name__ == "__main__":
    check_ollama_version()
    check_ollama_version('ollama')


import argparse
import yaml
import subprocess
import datetime
import re

parser = argparse.ArgumentParser(
    prog="python3 check_models.py",
    description="Before running check_models.py, please make sure you installed ollama successfully \
        on macOS, Linux, or WSL2 on Windows. You can check the website: https://ollama.com")

parser.add_argument("-v",
                    "--verbose",
                    action="store_true",
                    help="this program helps you check whether you have ollama benchmark models installed")

parser.add_argument("-m",
                    "--models",
                    type=str,
                    help="provide benchmark models YAML file path. ex. ../data/benchmark_models.yml")

parser.add_argument("-b",
                    "--benchmark",
                    type=str,
                    help="provide benchmark YAML file path. ex. ../data/benchmark1.yml")

parser.add_argument("-t",
                    "--test",
                    action="store_true",
                    help="run in test mode with minimal output")

def parse_yaml(yaml_file_path):
    with open(yaml_file_path, 'r') as stream:
        try:
            data = yaml.safe_load(stream)
        except yaml.YAMLError as e:
            print(e)
    return data

def parse_duration(duration_str):
    """ Parse duration from strings in various formats (s, ms, µs, m+s) to seconds as a float. """
    if 'ms' in duration_str:
        return float(duration_str.replace('ms', '')) / 1000.0
    elif 'µs' in duration_str:
        return float(duration_str.replace('µs', '')) / 1000000.0
    elif 's' in duration_str:
        match = re.match(r'(?:(\d+)m)?(\d*\.?\d*)s', duration_str)
        if match:
            minutes = int(match.group(1)) if match.group(1) else 0
            seconds = float(match.group(2)) if match.group(2) else 0
            return 60 * minutes + seconds
        else:
            raise ValueError(f"Unsupported duration format: {duration_str}")
    else:
        raise ValueError("Unsupported duration format")

def run_benchmark(models_file_path, steps, benchmark_file_path, is_test, ollamabin):
    models_dict = parse_yaml(models_file_path)
    benchmark_dict = parse_yaml(benchmark_file_path)
    allowed_models = {e['model'] for e in models_dict['models']}
    ans = {}

    num_steps = steps
    prompt_dict = benchmark_dict['prompts']

    if is_test:
        prompt_dict = benchmark_dict["testPrompts"]
        num_steps = min(len(prompt_dict), steps)

    for model in models_dict['models']:
        model_name = model['model']
        if model_name in allowed_models:
            loc_dt = datetime.datetime.today()
            total_tokens = 0
            total_eval_duration = 0.0
            total_duration = 0.0
            eval_durations = []
            tokens_per_second = []

            if not is_test:
                print(f"Starting evaluation for model: {model_name}\n")

            for index, prompt in enumerate(prompt_dict[:num_steps], start=1):
                prompt_text = prompt['prompt']
                if not is_test:
                    print(f"Evaluating prompt {index}/{num_steps}")
                    print('-' * 10)

                try:
                    result = subprocess.run(
                        [ollamabin, 'run', model_name, prompt_text, '--verbose'],
                        capture_output=True,
                        text=True,
                        check=True,
                        encoding='utf-8'
                    )
                    std_err = result.stderr or ''

                    for line in std_err.split('\n'):
                        if line.lstrip().startswith("prompt"):
                            continue
                        if 'eval count:' in line:
                            tokens = int(line.split()[-2])
                            total_tokens += tokens
                            if not is_test:
                                print(f"Produced Tokens: {tokens}")
                        if 'eval duration:' in line:
                            duration_str = line.split(':')[-1].strip()
                            eval_duration = parse_duration(duration_str)
                            eval_durations.append(eval_duration)
                            tokens_per_second.append(tokens / eval_duration)
                            total_eval_duration += eval_duration
                            if not is_test:
                                print(f"Decoding Seconds: {eval_duration:.3f}s")
                                print(f"Tokens/Second: {tokens / eval_duration:.3f}")
                        if 'total duration:' in line:
                            duration_str = line.split(':')[-1].strip()
                            duration = parse_duration(duration_str)
                            total_duration += duration
                            if not is_test:
                                print(f"Total Inference Seconds: {duration:.3f}s")
                except subprocess.SubprocessError as e:
                    print(f"Error during subprocess execution: {e}")

                if not is_test:
                    print('-' * 10 + "\n")

            if total_eval_duration > 0:
                average_eval_rate = total_tokens / total_eval_duration
                ans[model_name] = {
                    'average_token_per_second': f"{average_eval_rate:.3f}",
                    'total_tokens': total_tokens,
                    'total_decoding_seconds': f"{total_eval_duration:.3f}",
                    'total_inference_seconds': f"{total_duration:.3f}",
                }
                if not is_test:
                    print(f"Results for {model_name}: {ans[model_name]}")
                    print('-' * 10 + "\n")

    if is_test:
        print("Test run successful.")

    return ans

if __name__ == "__main__": 
    args = parser.parse_args()
    if args.models and args.benchmark:
        run_benchmark(args.models, 10, args.benchmark, args.test, 'ollama')
        if not args.test:
            print('-' * 40)



import argparse
import yaml
import subprocess
import datetime

parser = argparse.ArgumentParser(
    prog="python3 check_models.py",
    description="Before running check_models.py, please make sure you installed ollama successfully \
        on macOS, Linux, or WSL2 on Windows. You can check the website: https://ollama.ai",
    epilog="Author: Jason Chuang")

parser.add_argument("-v",
                    "--verbose",
                    action="store_true",
                    help="this program helps you check whether you have ollama benchmark models installed")

parser.add_argument("-m",
                    "--models",
                    type=str,
                    help="provide benchmark models YAML file path. ex. ../data/benchmark_models.yml")

def parse_yaml(yaml_file_path):
    with open(yaml_file_path, 'r') as stream:
        try:
            data=yaml.safe_load(stream)
            #print(d)
        except yaml.YAMLError as e:
            print(e)
    return data

if __name__ == "__main__": 
    args = parser.parse_args()
    #print(f"args.verbose value：{args.verbose}")
    if (args.models is not None):
        #print(f"args.models file path：{args.models}")
        models_dict = parse_yaml(args.models)
        
        loc_dt = datetime.datetime.today()
        # Writing to file
        with open(f'log_{loc_dt.strftime("%Y-%m-%d-%H%M%S")}.log', "w") as file1:
            for onemodel in models_dict['models']:
                model_name = onemodel['model']
                print(f'model_name =    {model_name}')
                file1.write(f'\nmodel_name =    {model_name}\n')
                
                result = subprocess.run(['ollama', 'run', model_name,f'"Why is the sky blue?"','--verbose'], capture_output=True, text=True, check=True)
                std_err = result.stderr
                #print(result.stderr)
                file1.write(std_err)
                
                for line in std_err.split('\n'):
                    if ('eval rate' in line) and ('prompt' not in line):
                        print(line)
                        number = float(line[-20:-8])
                        print(number)
                        
                
                print("-"*40)
                file1.write("\n"+"-"*40)
